@@ -14,13 +14,13 @@ class PowerML:
         self.current_model = "text-davinci-003"
 
     def predict(self,
-                prompt: str,
-                model: str = "",
+                prompt,
+                model: str = None,
                 stop: str = "",
                 max_tokens: int = 128,
                 temperature: int = 0,
                 ) -> str:
-        if model == "":
+        if model == "" or model is None:
             model = self.current_model
         logger.debug("Predict using model: " + model)
         # if the model is one of our models, then hit our api
@@ -35,8 +35,8 @@ class PowerML:
 
     def fit(self,
             data: list[str],
-            model: str = ""):
-        if model == "":
+            model: str = None):
+        if model == "" or model is None:
             model = self.current_model
         logger.debug("Fit using model: " + model)
         # Upload filtered data to train api
