@@ -3,24 +3,33 @@ import re
 from collections import Counter
 import random
 from tqdm import tqdm
-from transformers import GPT2Tokenizer
-from transformers.utils import logging
+import sys
+import os
 
 
 class PowerMLLearnTopics:
     def __init__(
             self,
             config={},
-            num_subsamples=20,
+            num_subsamples=87,
             sample_size=20,
+            max_output_tokens=256,
             create_model_name="unblocked/create-topics/v2",
             filter_model_name="unblocked/filter-topics"):
+        # This is a hack to silence printed message on import
+        sys.stdout = open(os.devnull, 'w')
+        from transformers import GPT2Tokenizer
+        from transformers.utils import logging
+        sys.stdout = sys.__stdout__
+
         self.model = PowerML(config)
         self.create_model_name = create_model_name
         self.filter_model_name = filter_model_name
         self.messages = []
         self.num_subsamples = num_subsamples
         self.sample_size = sample_size
+        self.max_output_tokens = max_output_tokens
+        self.memo_topics = {}
         self.memo_top_topics = {}
         self.memo_fltered_top_topics = {}
         self.tokenizer = GPT2Tokenizer.from_pretrained("gpt2")
@@ -34,8 +43,8 @@ class PowerMLLearnTopics:
         # If get_topics has been called on the same messages array
         # Then use the previous results
         hash_docs = hash(frozenset(self.messages))
-        if hash_docs in self.memo_top_topics:
-            return self.memo_top_topics[hash_docs]
+        if hash_docs in self.memo_topics:
+            return self.memo_topics[hash_docs]
 
         num_samples = len(self.messages)
         if self.num_subsamples * self.sample_size > num_samples:
@@ -44,31 +53,32 @@ class PowerMLLearnTopics:
                 self.messages) % self.sample_size else 0
 
         topic_type = 'one-word system components'
-        prompt = self.__prompt_format(topic_type)
-        # prompt_append = f'What are the {topic_type} that these messages discuss?\n\n1.'
-        num_samples = len(self.messages)
         sample_index = 0
         topic_counter = Counter()
-        for i in tqdm(range(self.num_subsamples)):
+        for _ in tqdm(range(self.num_subsamples)):
             if sample_index > num_samples:
                 break
             examples, sample_index = self.__subsample(
                 self.messages, sample_index, self.sample_size)
+            prompt = self.__prompt_format(topic_type)
             prompt = self.__add_examples(examples, prompt)
             output = self.model.predict(
-                prompt, max_tokens=256, temperature=0.7, model=self.create_model_name)
+                prompt, max_tokens=self.max_output_tokens, temperature=0.7, model=self.create_model_name)
             topics = self.__parse_output(output)
             topic_counter.update(topics)
             sorted_topics = self.__sort_topics(topic_counter)
+        topics = self.__get_topics(sorted_topics)
+        self.memo_topics[hash_docs] = topics
         top_topics = self.__get_top_topics(sorted_topics)
         self.memo_top_topics[hash_docs] = top_topics
-        return top_topics
+        return topics
 
     def get_filtered_topics(self):
         hash_docs = hash(frozenset(self.messages))
         if hash_docs in self.memo_fltered_top_topics:
             return self.memo_fltered_top_topics[hash_docs]
-        top_topics = self.get_topics()
+        _ = self.get_topics()
+        top_topics = self.memo_top_topics[hash_docs]
         not_topics = self.__filter_top_topics(top_topics)
         filtered_top_topics = list(set(top_topics) - set(not_topics))
         self.memo_fltered_top_topics[hash_docs] = filtered_top_topics
@@ -84,12 +94,12 @@ class PowerMLLearnTopics:
             prompt_dict[key] = ""
         return prompt_dict
 
-    def __add_examples(self, examples, prompt, max_total_tokens=4000, max_output_tokens=256):
+    def __add_examples(self, examples, prompt, max_total_tokens=4000):
         prompt_dict = prompt
         for i, example in enumerate(examples):
             key = "{{example" + str(i+1) + "}}"
             prompt_dict[key] = example
-            if self.__count_tokens(str(prompt_dict)) < max_total_tokens - max_output_tokens:
+            if self.__count_tokens(str(prompt_dict)) < max_total_tokens - self.max_output_tokens:
                 continue
             else:
                 break
@@ -114,8 +124,12 @@ class PowerMLLearnTopics:
     def __sort_topics(self, topic_counter):
         return sorted(topic_counter.items(), key=lambda x: x[1], reverse=True)
 
+    def __get_topics(self, sorted_topics):
+        # Return and save all topics
+        return [elt for elt, _ in sorted_topics]
+
     def __get_top_topics(self, sorted_topics):
-        # Return and save all that have more than count 1
+        # Return and save all topics that have more than count 1
         return [elt for elt, count in sorted_topics if count > 1]
 
     def __filter_top_topics(self, top_topics):
